#!/usr/bin/env python
"""
_ListThresholdsForSubmit_

Oracle implementation of ResourceControl.ListThresholdsForSubmit
"""




from WMCore.ResourceControl.MySQL.ListThresholdsForSubmit \
     import ListThresholdsForSubmit as MySQLListThresholdsForSubmit

class ListThresholdsForSubmit(MySQLListThresholdsForSubmit):
    pass
