"""
_CountFinishedSubscriptionsByWorkflow_

Oracle implementation of CountFinishedSubscriptionsByWorkflow
"""
from __future__ import division, print_function

from WMCore.WMBS.MySQL.Subscriptions.CountFinishedSubscriptionsByWorkflow \
    import CountFinishedSubscriptionsByWorkflow as MySQLCountFinishedSubscriptionsByWorkflow


class CountFinishedSubscriptionsByWorkflow(MySQLCountFinishedSubscriptionsByWorkflow):
    pass
