#!/usr/bin/env python
"""
_ListByState_

Oracle implementation of ListByState

Created on July 10, 2013

@author: tsarangi
"""

from WMCore.WMBS.MySQL.Jobs.ListByState import ListByState as MySQLListByState

class ListByState(MySQLListByState):
    pass
