#!/usr/bin/env python
"""
_ListSubsAndFilesetsFromWorkflow_

Oracle implementation of Subscriptions.ListSubsAndFilesetsFromWorkflow
"""
from __future__ import division

from WMCore.WMBS.MySQL.Subscriptions.ListSubsAndFilesetsFromWorkflow import ListSubsAndFilesetsFromWorkflow \
    as ListSubsAndFilesetsFromWorkflowMySQL


class ListSubsAndFilesetsFromWorkflow(ListSubsAndFilesetsFromWorkflowMySQL):
    pass
