# -*- coding: utf-8 -*-
"""
_SetPendingSlots_

Oracle implementation of Locations.SetPendingSlots
Created on Mon Jun 18 13:59:24 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Locations.SetPendingSlots import SetPendingSlots as MySQLSetPendingSlots

class SetPendingSlots(MySQLSetPendingSlots):
    """
    Same as MySQL version
    """
    pass
