#!/usr/bin/env python
"""
_MarkFinishedSubscriptions_

Oracle implementation of Subscriptions.MarkFinishedSubscriptions

Created on Aug 30, 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Subscriptions.MarkFinishedSubscriptions import MarkFinishedSubscriptions as MySQLMarkFinishedSubscriptions

class MarkFinishedSubscriptions(MySQLMarkFinishedSubscriptions):
    pass
