"""
__ListForSubmitter__

Oracle implementation of ListForSubmitter

Created on Apr 17, 2013

@author: dballest
"""

from WMCore.WMBS.MySQL.Workflow.ListForSubmitter import ListForSubmitter as MySQLListForSubmitter

class ListForSubmitter(MySQLListForSubmitter):
    pass
