#!/usr/bin/env python
"""
_GetByLFN_

Oracle implementation of GetByLFN
"""




from WMComponent.DBS3Buffer.MySQL.DBSBufferFiles.GetByLFN import GetByLFN as MySQLGetByLFN

class GetByLFN(MySQLGetByLFN):
    """

    Oracle implementation of GetByLFN

    """
    pass
