#!/usr/bin/env python
"""
_GetFilesForParentlessMerge_

Oracle implementation of Subscription.GetFilesForParentlessMerge
"""

from WMCore.WMBS.MySQL.Subscriptions.GetFilesForParentlessMerge import GetFilesForParentlessMerge as GetFilesForParentlessMergeMySQL

class GetFilesForParentlessMerge(GetFilesForParentlessMergeMySQL):
    pass
