#!/usr/bin/env python
"""
_FailOrphanFiles_

Oracle implementation of Subscriptions.FailOrphanFiles
"""

from WMCore.WMBS.MySQL.Subscriptions.FailOrphanFiles import FailOrphanFiles as MySQLFailOrphanFiles

class FailOrphanFiles(MySQLFailOrphanFiles):
    pass
