#!/usr/bin/env python
"""
_GetSpecAndNameFromTask_

Oracle implementation of Workflow.GetSpecAndNameFromTask

Created on Oct 9, 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Workflow.GetSpecAndNameFromTask import GetSpecAndNameFromTask as MySQLGetSpecAndNameFromTask

class GetSpecAndNameFromTask(MySQLGetSpecAndNameFromTask):
    pass
